package JCAT;

import java.awt.*;
import java.util.*;
import java.io.*;


class JavaFilter implements FilenameFilter 
{
    public boolean accept(File dir, String file)
    {
	return file.endsWith(".java");
    }
}


class JCAT extends Frame implements Version
{
    private J2SBox j2sBox;
    private PanCompBox pcBox;
    private PanExecBox peBox;
    private DirBox dirBox;

    private Vector fileVect;
  
    private String selectedFiles = "";
  
    private Traceable tracer_;
    private Analysis analyzer_;

    private Runtime rt_;

    public JCAT() {
    
	super(version_);

	//{{INIT_MENUS
	MenuBar mb = new MenuBar();
	fileMenu = new Menu("File");
	fileMenu.add(fileOpen = new MenuItem("Open..."));
	fileMenu.add(fileClose = new MenuItem("Close"));
	fileMenu.add(fileExit = new MenuItem("Exit"));
	mb.add(fileMenu);
	optionsMenu = new Menu("Options");
	optionsMenu.add(optionsJava2Spin = new MenuItem("Java2Spin"));
	optionsMenu.add(optionsPanCompilation = new MenuItem("Pan compilation"));
	optionsMenu.add(optionsPanExecution = new MenuItem("Pan execution"));
	optionsMenu.add(directoryMenu = new MenuItem("Directories"));
	mb.add(optionsMenu);
	runMenu = new Menu("Run");
	runMenu.add(runAnalysis = new MenuItem("Analysis"));
	runAnalysis.disable();
	runMenu.add(runSimulation = new MenuItem("Simulation"));
	runSimulation.disable();
	mb.add(runMenu);
	menuHelp = new Menu("Help");
	menuHelp.add(helpAbout = new MenuItem("About ..."));
	mb.add(menuHelp);
	setMenuBar(mb);
	//}}
    
	//{{INIT_CONTROLS
	setLayout(null);
	addNotify();
	resize(insets().left + insets().right + 602, insets().top + insets().bottom + 400);

	footLabel=new Label("Ready");
	add(footLabel);
	footLabel.reshape(insets().left + 6,insets().top + 364, 382, 13);
	footLabel.setFont(new Font("Dialog", Font.BOLD, 10));

	editOut = new TextArea(15,95);
	editOut.setFont(new Font("Dialog", Font.BOLD, 10));
	add(editOut);
	editOut.reshape(insets().left + 4,insets().top + 37,586,220);

	editErr = new TextArea(5, 95);
	editErr.setFont(new Font("Dialog", Font.BOLD, 10));
	add(editErr);
	editErr.reshape(insets().left + 4,insets().top + 267,586,88);

	stepButton=new Button("Step");
	stepButton.disable();
	add(stepButton);
	stepButton.reshape(insets().left + 508,insets().top + 364,80,26);
	quitButton = new Button("Quit");
	quitButton.disable();
	add(quitButton);
	quitButton.reshape(insets().left + 408, insets().top + 364, 80, 26);
	filesLabel=new Label("Files:");
	filesLabel.setFont(new Font("Dialog",Font.BOLD,10));
	add(filesLabel);
	filesLabel.reshape(insets().left + 8,insets().top + 8,40,25);
	selectedLabel=new Label("");
	selectedLabel.setFont(new Font("Dialog",Font.BOLD,10));
	add(selectedLabel);
	selectedLabel.reshape(insets().left + 62,insets().top + 7,724,26);
	//}}

	j2sBox = new J2SBox(this);
	pcBox  = new PanCompBox(this);
	peBox  = new PanExecBox(this);
	dirBox = new DirBox(this);

	fileVect = new Vector();
	rt_ = Runtime.getRuntime();

	show();
    }


    public synchronized void show() {
	move(50, 50);
	super.show();
    }
  

    public boolean handleEvent(Event event) {
	if (event.id == Event.ACTION_EVENT && event.target == stepButton) {
	    clickedStepButton();
	    return true;
	}
	else       
	    if (event.id == Event.ACTION_EVENT && event.target == quitButton) {
		clickedQuitButton();
		return true;
	    }
	    else
		if (event.id == Event.WINDOW_DESTROY) {
		    hide();         // hide the Frame
		    dispose();      // tell windowing system to free resources
		    System.exit(0); // exit
		    return true;
		}
    
	return super.handleEvent(event);
    }

  
    public boolean action(Event event, Object arg) {
	if (event.target instanceof MenuItem) {
	    String label = (String) arg;
	    if (event.target == directoryMenu) {
		selectedDirectoryMenu();
		return true;
	    } else if (event.target == fileExit) {
		selectedFileExit();
		return true;
	    } else if (event.target == helpAbout) {
		selectedHelpAbout();
		return true;
	    } else if (event.target == runAnalysis) {
		selectedRunAnalysis();
		return true;
	    } else if (event.target == runSimulation) {
		selectedRunSimulation();
		return true;
	    } else if (event.target == optionsPanExecution) {
		selectedOptionsPanExecution();
		return true;
	    } else if (event.target == optionsPanCompilation) {
		selectedOptionsPanCompilation();
		return true;
	    } else if (event.target == optionsJava2Spin) {
		selectedOptionsJava2Spin();
		return true;
	    } else if (event.target == fileOpen) {
		selectedFileOpen();
		return true;
	    } else if (event.target == fileClose) {
		selectedFileClose();
		return true;
	    }
	}
    
	return super.action(event, arg);
    }
  

    public static void main(String args[]) 
    {      
	if (!Configure.checkConfiguration()) {
	    String s[] = {"Invalid configuration      ",
			  "Check out shell script jcat",
			  "Sorry, giving up ...       "};
	    Configure.warning(s);
	}
	else 
	    new JCAT();
    }


    Menu fileMenu;
    MenuItem fileOpen;
    MenuItem fileClose;
    MenuItem fileExit;
    Menu optionsMenu;
    MenuItem optionsJava2Spin;
    MenuItem optionsPanCompilation;
    MenuItem optionsPanExecution;
    MenuItem directoryMenu;
    Menu runMenu;
    MenuItem runAnalysis;
    MenuItem runSimulation;
    Menu menuHelp;
    MenuItem helpAbout;

    Label footLabel;
    TextArea editOut;
    TextArea editErr;
    Button stepButton;
    Button quitButton;
    Label filesLabel;
    Label selectedLabel;

    FileDialog fileOpenBox;
  
    public void selectedFileOpen() {
	String dir, file;
    
	fileOpenBox = new FileDialog(this, "Open...");
	fileOpenBox.setFilenameFilter(new JavaFilter());
	fileOpenBox.setFile("*.java");
	fileOpenBox.show();
    
	dir  = fileOpenBox.getDirectory();
	file = fileOpenBox.getFile();

	if (file == null)
	    return;

	dir = (dir == null) ? "./" : dir;
	String path = dir + file;

	if (file.endsWith(".java")) {
	    fileVect.addElement(path);
	} else 
	    if (file.endsWith(".class") && Configure.jadAvail) {
		Decompiler dcp = new Decompiler(dirBox, editOut, editErr, 
						footLabel, dir, path);
		
		if (dcp.decompile()) {	       
		    file = file.substring(0, file.indexOf(".class")) + ".jad";
		    path = (dir == null) ? "" : dir + file;		
		    fileVect.addElement(path);      
		}
	    } 
	    else {
		editErr.appendText(file + ": invalid suffix\n");
		return;
	    }

	if (file != null) {
	    selectedFiles += file + " ";
	    selectedLabel.setText(selectedFiles);
	}
    
	runAnalysis.enable();
	runSimulation.enable();
    }


    public void selectedFileClose() {
	fileVect.removeAllElements();
	selectedFiles = "";
	selectedLabel.setText("");
	editOut.setText("");
	editErr.setText("");
	footLabel.setText("Ready");
	runAnalysis.disable();
	runSimulation.disable();
    }

  
    public void selectedOptionsJava2Spin() {
	j2sBox.show();
    }

  
    public void selectedOptionsPanCompilation() {
	pcBox.show();
    }

  
    public void selectedOptionsPanExecution() {
	peBox.show();
    }

  
    public void selectedRunAnalysis() {
	String files = " ";
    
	for (int next = 0; next < fileVect.size(); next ++)
	    files += " " + (String)fileVect.elementAt(next);
    
	analyzer_ = new Analysis(dirBox, j2sBox.getOptions() + files, 
				 pcBox.getOptions(), peBox.getOptions(), 
				 editOut, editErr, footLabel);
	editOut.setText("");
	editErr.setText("");

	switch(analyzer_.getResult()) {

	case Analysis.ASSERTION_VIOLATED:
	    if ((new YesNo("Assertion violated. Follow trail?", 
			   this)).getState() == YesNo.OK) 
		{
		    tracer_ = new Debug(dirBox, editOut, editErr, footLabel);
	
		    tracer_.makeTrail();
		    stepButton.enable();
		    quitButton.enable();
		}

	    break;
	
	case Analysis.INVALID_ENDSTATE:
	    if ((new YesNo("Invalid end state. Follow trail?", 
			   this)).getState() == YesNo.OK) 
		{
		    tracer_ = new Debug(dirBox, editOut, editErr, footLabel);

		    tracer_.makeTrail(); 
		    stepButton.enable();
		    quitButton.enable();
		}
	}
    }

  
    public void selectedRunSimulation() {    
	String files = " ";
    
	for (int next = 0; next < fileVect.size(); next ++)
	    files += " " + (String)fileVect.elementAt(next);
    
	tracer_ = new Simulation(dirBox, j2sBox.getOptions() + files, 
				 editOut, editErr, footLabel);
    
	editOut.setText("");
	editErr.setText("");

	tracer_.makeTrail();
	stepButton.enable();
	quitButton.enable();
    }

  
    public void selectedHelpAbout() {
    }

  
    public void selectedFileExit() {
	hide();
	dispose();
	System.exit(0);
    }
  

    public void selectedDirectoryMenu() {
	dirBox.show();
    }


    public void clickedQuitButton() {

	if (tracer_ != null)
	    tracer_.quit();
    
	quitButton.disable();
	stepButton.disable();
	editErr.appendText("\nQuit!\n");
	footLabel.setText("Ready");
    }


    public void clickedStepButton() {
	if (!tracer_.nextStep()) {

	    quitButton.disable();
	    stepButton.disable();

	    if ((new YesNo("Run once more time?", this)).getState() == 
		YesNo.OK) {
		stepButton.enable();
		quitButton.enable();
		tracer_.init();
	    }
	    else {	
		tracer_.clear();
		tracer_ = null;
	
		stepButton.disable();
		quitButton.disable();

		editOut.setText("");
		editErr.setText("");
	
		footLabel.setText("Ready");
	    }
	}
    }
}
